# -*- coding: utf-8 -*-
import hashlib
import json
import math
import os
import re
import string
import unicodedata
from collections import OrderedDict
from contextlib import contextmanager
from datetime import datetime

import pyqrcode
import requests
import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin
import xbmcvfs
from dateutil.parser import parse

from resources.lib.const import STRINGS, SETTINGS, genre_lang, GENERAL, lang_code_gui, URL, alphabet, lang_countries, \
    common_chars, numbers
from resources.lib.utils.qrcode import QRCode
from resources.lib.vendor.md5crypt import md5crypt

try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

ADDON = xbmcaddon.Addon()


@contextmanager
def busy_dialog():
    xbmc.executebuiltin('ActivateWindow(busydialognocancel)')
    try:
        yield
    finally:
        xbmc.executebuiltin('Dialog.Close(busydialognocancel)')


def get_icon(name):
    return os.path.join(ADDON.getAddonInfo('path'), 'resources', 'skins', 'images', 'icons', name)


def notification(header, message, time=5000, icon=ADDON.getAddonInfo('icon'), sound=True):
    xbmcgui.Dialog().notification(header, message, icon, time, sound)


def show_settings():
    ADDON.openSettings()


def to_utf8_string(val):
    if not isinstance(val, str):
        val = val.encode('utf8')
    return str(val)


def strip_accents(s):
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))


# You must end directory in order to update container
def go_to_plugin_url(url):
    xbmc.executebuiltin('Container.Update("%s")' % url)


# You must end directory in order to update container
def replace_plugin_url(url):
    xbmc.executebuiltin('Container.Update("%s", "replace")' % url)


def refresh():
    xbmc.executebuiltin('Container.Refresh')


def current_path():
    return xbmc.getInfoLabel('Container.FolderPath')


def set_resolved_url(handle, li):
    xbmcplugin.setResolvedUrl(handle, True, li)


def show_input(heading, input_type=xbmcgui.INPUT_ALPHANUM, **kwargs):
    return xbmcgui.Dialog().input(heading, type=input_type, **kwargs)


def get_info(info):
    return ADDON.getAddonInfo(info)


def get_settings(setting):
    return ADDON.getSetting(setting).strip()


def set_settings(setting, value):
    ADDON.setSetting(setting, str(value))


# It seems a bug in python https://bugs.python.org/issue27400
def parse_date(string_date):
    return parse(string_date, ignoretz=True)


def get_current_datetime_str():
    return datetime.now().strftime(STRINGS.DATETIME)


def datetime_from_iso(iso_date):
    return parse_date(iso_date)


def get_setting_as_datetime(setting):
    try:
        return parse_date(get_settings(setting))
    except:
        return None


def get_setting_as_bool(setting):
    return ADDON.getSettingBool(setting)


def get_setting_as_float(setting):
    try:
        return float(get_settings(setting))
    except ValueError:
        return 0


def get_setting_as_int(setting):
    try:
        return int(get_setting_as_float(setting))
    except ValueError:
        return 0


def get_string(string_id):
    return ADDON.getLocalizedString(string_id).encode('utf8')


def translate_string(s):
    regex = re.compile(r'\${(\d+)}', re.S)
    return regex.sub(lambda m: m.group().replace(m.group(), get_string(int(m.group()[2:-1])), 1), s)


def get_kodi_version():
    return xbmc.getInfoLabel('System.BuildVersion')


def get_screen_width():
    return xbmc.getInfoLabel('System.ScreenWidth')


def get_os_version():
    return xbmc.getInfoLabel('System.BuildVersion')


def get_screen_height():
    return xbmc.getInfoLabel('System.ScreenHeight')


def convert_size(size_bytes):
    if size_bytes == 0 or size_bytes is None:
        return "0 B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])


def convert_bitrate(mbit):
    if mbit == 0 or mbit is None:
        return "0 Mbit/s"
    p = math.pow(1024, 2)
    s = round(mbit / p, 2)
    return "%s %s" % (s, "Mbit/s")


def make_table(matrix):
    matrix_length = len(matrix)
    if matrix_length == 0:
        return matrix
    for i in range(len(matrix[0])):
        longest = len(matrix[0][i])
        for r in range(matrix_length):
            length = len(matrix[r][i])
            if length > longest:
                longest = length

        for j, strings in enumerate(matrix):
            string = strings[i]
            length = len(string)
            diff = longest - length
            spaces = "" if length > 0 else ' '
            for r in range(diff):
                spaces += STRINGS.TABLE_SPACES if length > 0 else ' '

            matrix[j][i] = spaces + string
    return matrix


def append_list_items_to_nested_list_items(_list, list_to_append):
    for i, nested_list in enumerate(_list):
        nested_list.append(list_to_append[i])
    return _list


def user_agent():
    return xbmc.getUserAgent()


def common_headers():
    return {
        'User-Agent': user_agent(),
        'X-Uuid': get_settings(SETTINGS.UUID)
    }


def delete_try(obj, key):
    try:
        del obj[key]
    except:
        pass


def apply_strings(text, *args):
    args = list(args)
    res = args.pop(0)
    for string in args:
        res = res.format(string)
    return res.format(*text)


def time_limit_expired(settings, limit):
    last_version_check = get_setting_as_datetime(settings)
    current_datetime = datetime.now()
    if last_version_check is None:
        return True
    return current_datetime - limit > last_version_check


def clear_kodi_addon_cache():
    xbmc.executebuiltin("UpdateLocalAddons")
    xbmc.executebuiltin('UpdateAddonRepos')


def hash_password(password, salt):
    """Creates password hash with salt from Webshare API"""
    return hashlib.sha1(md5crypt(password, salt=salt).encode('utf-8')).hexdigest()


def get_application_name():
    cmd = ('{"jsonrpc":"2.0", "method":"Application.GetProperties",'
           '"params": {"properties": ["name"]}, "id":1}')
    data = json.loads(xbmc.executeJSONRPC(cmd))
    if "result" in data and "name" in data["result"]:
        return data["result"]["name"]
    else:
        raise ValueError


def get_log_file_path():
    app_name = get_application_name().lower()
    return xbmc.translatePath('special://logpath') + "{0}.log".format(app_name)


def get_kodi_language():
    return xbmc.getLanguage(xbmc.ISO_639_1)


def is_playing():
    return xbmc.Player().isPlaying()


def read_log():
    try:
        lf = xbmcvfs.File(get_log_file_path())
        sz = lf.size()
        if sz > 2000000:
            return False, 'log file is too large'
        content = lf.read()
        lf.close()
        if content:
            return True, content
        else:
            return False, 'log file is empty'
    except:
        return False, 'unable to read log file'


def clean_log(content):
    replaces = (('//.+?:.+?@', '//USER:PASSWORD@'), ('<user>.+?</user>', '<user>USER</user>'),
                ('<pass>.+?</pass>', '<pass>PASSWORD</pass>'),)
    for pattern, repl in replaces:
        content = re.sub(pattern, repl, content)
        return content


def post_log(data):
    try:
        response = requests.post(URL.LOG_UPLOAD_URL, data=data)
        if 'key' in response.json():
            result = 'https://paste.kodi.tv/' + response.json()['key']
            return True, result
        elif 'message' in response.json():
            # upload failed, paste may be too large
            return False, response.json()['message']
        else:
            return False, response.text
    except requests.exceptions.RequestException as e:
        return False, str(e)


def show_log_dialog(url):
    cwd = ADDON.getAddonInfo('path').decode('utf-8')
    profile = ADDON.getAddonInfo('profile').decode('utf-8')
    image_file = os.path.join(xbmc.translatePath(profile), '%s.png' % str(url.split('/')[-1]))
    qr_img = pyqrcode.create(url)
    qr_img.png(image_file, scale=10)
    qr = QRCode("sc2-loguploader.xml", cwd, "default", image=image_file, text=url)
    qr.doModal()
    del qr
    xbmcvfs.delete(image_file)


def translate_genres(genre_list):
    translated_list = []
    for genre in genre_list:
        lang_id = genre_lang.get(genre)
        translated_genre = get_string(lang_id) if lang_id else genre
        translated_list.append(translated_genre.decode('utf-8'))
    return translated_list


def spinner_start():
    xbmc.executebuiltin("ActivateWindow(busydialog)")


def spinner_stop():
    xbmc.executebuiltin("Dialog.Close(busydialog)")


def languages_gui(languages):
    return [lang_code_gui.get(language) for language in languages]


def kodi_json_request(message, command_name, command_params):
    params = {'sender': GENERAL.PLUGIN_ID,
              'message': message,
              'data': {'command': command_name,
                       'command_params': command_params
                       },
              }
    data = json.dumps({'jsonrpc': '2.0',
                       'method': 'JSONRPC.NotifyAll',
                       'params': params,
                       'id': 1,
                       })
    xbmc.executeJSONRPC(data)


def data_dir():
    """"get user data directory of this addon.
    according to http://wiki.xbmc.org/index.php?title=Add-on_Rules#Requirements_for_scripts_and_plugins
    """
    datapath = xbmc.translatePath(ADDON.getAddonInfo('profile'))
    if not xbmcvfs.exists(datapath):
        xbmcvfs.mkdir(datapath)
    return datapath


def colorize(color, text):
    return STRINGS.COLOR.format(color=color, text=text)


def get_current_alphabet_index(text):
    return get_alphabet_index(get_kodi_language(), text)


def get_lang_countries(langs):
    countries = []
    for lang in langs:
        countries += lang_countries.get(lang, [])
    return list(OrderedDict.fromkeys(countries))


def atoi(text):
    return int(text) if text.isdigit() else text


def get_alphabet_index(lang_code, text):
    a = common_chars + numbers + [c.decode('utf-8') for c in alphabet.get(lang_code, string.ascii_lowercase)]
    text = text.decode('utf-8').lower()
    index = ()
    while len(text) > 0:
        # digits = ''
        # while len(text) > 0 and text[0] in numbers:
        #     digits += text[0]
        #     text = text[1:]
        # if digits != '':
        #     index += ((0, int(digits)),)

        while len(text) > 0:
            match = ''
            match_i = -1
            for i, char in enumerate(a):
                if char == text[:len(char)] and len(char) > len(match):
                    match = char
                    match_i = i
            index += ((1, match_i),)
            text = text[len(match) if match_i >= 0 else 1:]
    return index


def key_combiner(*key_funcs):
    def helper(elem):
        return [key_func(elem) for key_func in key_funcs]

    return helper


def is_youtube_url(url):
    youtube_regex = (
        r'(https?://)?(www\.)?'
        '(youtube|youtu|youtube-nocookie)\.(com|be)/'
        '(watch\?v=|embed/|v/|.+\?v=)?([^&=%\?]{11})')

    youtube_regex_match = re.match(youtube_regex, url)
    if youtube_regex_match:
        return youtube_regex_match

    return youtube_regex_match
